// src/lib/create-params.svelte.ts
import { tick } from "svelte";

// src/lib/adapters/browser.ts
function browser(options = {}) {
  const { windowObj = window, replace = false } = options;
  const replaceState = windowObj.history.replaceState.bind(windowObj.history);
  const pushState = windowObj.history.pushState.bind(windowObj.history);
  const update = replace ? replaceState : pushState;
  return {
    isBrowser: () => typeof window !== "undefined",
    browser: {
      read: () => windowObj.location,
      save: (search, hash) => update(null, "", `${search}${hash}`)
    },
    server: {
      save: () => {
      }
    }
  };
}

// src/lib/utils.ts
import { parse } from "valibot";
function parseSearchString(search) {
  const params = new URLSearchParams(search);
  return Object.fromEntries(params.entries());
}
function objectToQueryString(init) {
  return `?${new URLSearchParams(init)}`;
}
function parseObject(schemas, input) {
  return Object.fromEntries(
    Object.entries(schemas).map(([key, schema]) => [
      key,
      parseValue(key, schema, input[key])
    ])
  );
}
function parseValue(key, schema, value) {
  if (typeof schema === "function") return schema(value);
  if (isZodSchema(schema)) return schema.parse(value);
  if (isValibotSchema(schema)) return parse(schema, value);
  const name = schema.constructor.name;
  throw new Error(
    `Unknown validator type (${name}) for param "${key}" (value: ${value})`
  );
}
function isZodSchema(obj) {
  return typeof obj === "object" && obj && "parse" in obj && "safeParse" in obj && typeof obj.parse === "function" && typeof obj.safeParse === "function";
}
function isValibotSchema(obj) {
  return typeof obj === "object" && obj && "async" in obj && "kind" in obj && "_run" in obj && obj.kind === "schema" && typeof obj.async === "boolean" && typeof obj._run === "function";
}
function parseQueryParams(params, schemas) {
  if (typeof schemas === "function") return schemas(params);
  if (isZodSchema(schemas)) return schemas.parse(params);
  if (isValibotSchema(schemas)) return parse(schemas, params);
  return parseObject(schemas, params);
}
function mapValues(object, mapFn) {
  return Object.fromEntries(
    Object.entries(object).map(([key, value]) => [key, mapFn(value)])
  );
}
function debounce(func, delay) {
  if (delay === 0) return func;
  let timeoutId;
  return (...args) => {
    clearTimeout(timeoutId);
    timeoutId = setTimeout(() => {
      func(...args);
    }, delay);
  };
}
function diff(obj1, obj2) {
  const keys1 = Object.keys(obj1);
  const keys2 = Object.keys(obj2);
  if (keys1.length !== keys2.length) {
    return true;
  }
  for (const key of keys1) {
    if (obj1[key] !== obj2[key]) {
      return true;
    }
  }
  return false;
}

// src/lib/create-params.svelte.ts
function createUseQueryParams(validators, options = {}) {
  const {
    debounce: delay = 0,
    windowObj = typeof window === "undefined" ? void 0 : window,
    adapter = browser({ windowObj }),
    serialise = (value) => typeof value === "string" ? value : JSON.stringify(value)
  } = options;
  let raw = $state({});
  const query = $derived(parseQueryParams(raw, validators));
  const merged = $derived({ ...raw, ...query });
  function readFromBrowser() {
    raw = parseSearchString(adapter.browser.read().search);
  }
  const persistToBrowser = debounce((search, hash) => {
    return tick().then(() => adapter.browser.save(search, hash));
  }, delay);
  function persistParams() {
    const search = objectToQueryString(raw);
    adapter.isBrowser() ? persistToBrowser(search, adapter.browser.read().hash) : adapter.server.save(search);
  }
  let unsubscribe;
  if (windowObj) {
    unsubscribe = addWindowListener(windowObj, readFromBrowser);
  }
  return function useQueryParams(url) {
    raw = parseSearchString(url.search);
    const params = {};
    for (const key of Object.keys(query)) {
      Object.defineProperty(params, key, {
        enumerable: true,
        configurable: true,
        get() {
          return query[key];
        },
        set(newValue) {
          const value = serialise(newValue);
          if (value !== raw[key]) {
            raw = { ...raw, [key]: value };
            persistParams();
          }
        }
      });
    }
    if (typeof window !== "undefined" && !unsubscribe) {
      unsubscribe = addWindowListener(window, readFromBrowser);
    }
    return [
      params,
      {
        get raw() {
          return raw;
        },
        get search() {
          return objectToQueryString(raw);
        },
        get all() {
          return merged;
        },
        keys() {
          return Object.keys(query);
        },
        entries() {
          return Object.entries(query).map(([key]) => [key, query[key]]);
        },
        set(params2) {
          const updated = mapValues(params2, serialise);
          if (diff(raw, updated)) {
            raw = updated;
            persistParams();
          }
        },
        update(params2) {
          for (const [key, newValue] of Object.entries(params2)) {
            const value = serialise(newValue);
            if (value !== raw[key]) {
              raw = { ...raw, ...mapValues(params2, serialise) };
              persistParams();
              break;
            }
          }
        },
        remove(...params2) {
          raw = Object.fromEntries(
            Object.entries(raw).filter(([key]) => !params2.includes(key))
          );
        },
        unsubscribe() {
          return unsubscribe?.();
        }
      }
    ];
  };
}
function addWindowListener(windowObj, update) {
  const replaceState = windowObj.history.replaceState.bind(windowObj.history);
  const pushState = windowObj.history.pushState.bind(windowObj.history);
  windowObj.history.replaceState = (...args) => {
    replaceState(...args);
    update();
  };
  windowObj.history.pushState = (...args) => {
    pushState(...args);
    update();
  };
  windowObj.addEventListener("popstate", update);
  return () => {
    windowObj.removeEventListener("popstate", update);
    windowObj.history.pushState = pushState;
    windowObj.history.replaceState = replaceState;
  };
}
export {
  createUseQueryParams
};
